// CCLSTLMap.cpp : Implementation of CCCLSTLMap
#include "stdafx.h"
#include "CCL.h"
#include "CCLSTLMap.h"
#include "CCLUtility.h"

/////////////////////////////////////////////////////////////////////////////
// CCCLSTLMap

#include "StreamHelpers.h"
#include <comdef.h>
#import "CCL.tlb" rename_namespace("CCL")

STDMETHODIMP CCCLSTLMap::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ICCLSTLMap,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}
// IPersistStream implementation

STDMETHODIMP CCCLSTLMap::IsDirty()
{
	return S_OK;
}

STDMETHODIMP CCCLSTLMap::Load(IStream* pStm)
{
	// Stream in the file version
	short versionMajor;
	short versionMinor;
	try
	{
		*pStm >> versionMajor;
		*pStm >> versionMinor;
		if( m_versionMajor == versionMajor && m_versionMinor == versionMinor)
		{
			// TODO: Load your stuff here
			long size;  
			*pStm >> size;
			for (long i=0; i<size; i++ )
			{
				_bstr_t key;
				CComVariant value;

				*pStm >> key;			
				*pStm >> value;

				m_contents.insert(ContainerType::value_type(key,value));
			}
			
			return S_OK;
		}
		else
			return E_FAIL;
	}
	catch(_com_error& e)
	{
		return e.Error();
	}
	return E_FAIL;
}

STDMETHODIMP CCCLSTLMap::Save(IStream* pStm, BOOL fClearDirty)
{
	try
	{
		*pStm << m_versionMajor; 
		*pStm << m_versionMinor; 
// TODO: Save your stuff here
		long size = m_contents.size();  
		*pStm << size;

		ContainerTypeIterator	it;	
		for (it=m_contents.begin(); it != m_contents.end(); it++ )
		{
			*pStm << (*it).first;
			*pStm << (*it).second;
		}
		return S_OK;
	}
	catch(_com_error& e)
	{
		return e.Error();
	}
}

STDMETHODIMP CCCLSTLMap::GetSizeMax(ULARGE_INTEGER* pcbSize)
{
	pcbSize->QuadPart = sizeof( m_versionMajor ) +
						sizeof( m_versionMinor );		
	long size = m_contents.size();
	pcbSize->QuadPart += sizeof( size  ) + 
						size * sizeof(ContentType);
	ContainerTypeIterator it;
	for(it = m_contents.begin();it != m_contents.end(); it++)
	{
		pcbSize->QuadPart += (*it).first.length();
		pcbSize->QuadPart += getVariantMarshalSize((*it).second);		// TODO: 
																// Code assumes ContentType is a VARIANT
																// You will have to calcualte your size here
																// for maps you will have to add the size of both items
	}
	return S_OK;
}

STDMETHODIMP CCCLSTLMap::GetClassID(CLSID *pClassID)
{
	*pClassID = GetObjectCLSID();
	return S_OK;
}	

STDMETHODIMP CCCLSTLMap::get__NewEnum(LPUNKNOWN * pVal)
{
	if (!pVal) return E_POINTER;
	*pVal = 0;
	typedef CComObject< CComEnum<IEnumVARIANT, &IID_IEnumVARIANT, VARIANT,_Copy<VARIANT> > > CComEnumVARIANT;
	VARIANT *pav = new VARIANT[m_contents.size()];
	int j = 0;
	try
	{
		for (ContainerTypeIterator i = m_contents.begin();i != m_contents.end(); ++i)
		{
			::VariantInit(&pav[j]);
			// TODO: write your own conversion here if necessary
			CComVariant	v = (BSTR)(*i).first;		// assume that *i can be converted to a variant
			v.Detach(&pav[j++]);
		}                
	}
	catch (const _com_error& x)                
	{
		for (int i = 0; i < (j-1); ++i)                        
		{
			::VariantClear(&pav[i]);
		}
		delete [] pav;
		return x.Error();                
	}
	CComEnumVARIANT *pEnum = 0;
	HRESULT hr = CComEnumVARIANT::CreateInstance(&pEnum);
	if (hr == S_OK)                
	{
		hr = pEnum->Init(&pav[0], &pav[m_contents.size()], 0, AtlFlagTakeOwnership);
		if (hr == S_OK)                        
		{
			hr = pEnum->QueryInterface(IID_IUnknown, (void **) pVal);
		}                
	}
	return hr;        
}

HRESULT 
CCCLSTLMap::bounds_check(BSTR key,ContainerTypeIterator& it)
{
	it = m_contents.find(key);
	if(it == m_contents.end())
	{
		_variant_t i(key);
		i.ChangeType(VT_BSTR);
		_bstr_t msg("CCCLSTLMap[");
		msg += _bstr_t(i);
		msg+="]: no such entry.";
		return AtlReportError(	GetObjectCLSID(), 
								(const char*)msg,
								IID_ICCLSTLMap,
								E_INVALIDARG);
	}
	return S_OK;
}
STDMETHODIMP CCCLSTLMap::get_Value(BSTR key, VARIANT * pVal)
{
	return get_at(key,pVal);
}

STDMETHODIMP CCCLSTLMap::put_Value(BSTR key, VARIANT newVal)
{
	return put_at(key,newVal);
}

STDMETHODIMP CCCLSTLMap::get_Count(long * pVal)
{
	return size(pVal);
}



STDMETHODIMP CCCLSTLMap::size(long * sz)
{
	if(sz == NULL) return E_POINTER;
	*sz = m_contents.size();
	return S_OK;
}

STDMETHODIMP CCCLSTLMap::max_size(long * max)
{
	if(max == NULL) return E_POINTER;
	*max = m_contents.max_size();

	return S_OK;
}

STDMETHODIMP CCCLSTLMap::clear()
{
	m_contents.clear();
	return S_OK;
}

STDMETHODIMP CCCLSTLMap::erase(BSTR key)
{
	m_contents.erase(key);
	return S_OK;
}

STDMETHODIMP CCCLSTLMap::find(BSTR key, VARIANT * value)
{
	if(value == NULL) return E_POINTER;
	ContainerTypeIterator iterator;
	HRESULT hr = bounds_check(key,iterator);
	if(FAILED(hr))
		return hr;
	::VariantCopy(value,&(*iterator).second);

	return S_OK;
}

STDMETHODIMP CCCLSTLMap::empty(BOOL* emp)
{
	if(emp == NULL) return E_POINTER;
	*emp = m_contents.empty();

	return S_OK;
}

STDMETHODIMP CCCLSTLMap::insert(BSTR key, VARIANT value)
{
	_variant_t v(value);

	std::pair<ContainerTypeIterator, bool> insertion =
	m_contents.insert(ContainerType::value_type(key,value));

	if(! insertion.second)	// the value already exists we will overwrite it
	{
		(*insertion.first).second = value;
	}

	return S_OK;
}

STDMETHODIMP CCCLSTLMap::get_at(BSTR key, VARIANT * pVal)
{
	return find(key,pVal);
}

STDMETHODIMP CCCLSTLMap::put_at(BSTR key, VARIANT newVal)
{
	return insert(key,newVal);
}


STDMETHODIMP CCCLSTLMap::exists(BSTR key, BOOL* found)
{
	if(found == NULL)	return E_POINTER;

	ContainerTypeIterator iterator = m_contents.find(key);
	*found = (iterator == m_contents.end()) ? false : true;
	return S_OK;
}

STDMETHODIMP CCCLSTLMap::extract(BSTR key, VARIANT * value, BOOL * found)
{
	if(value == NULL) return E_POINTER;
	if(found == NULL) return E_POINTER;

	ContainerTypeIterator it = m_contents.find(key);
	if(it == m_contents.end())
	{
		*found = false;
		return S_OK;
	}

	::VariantCopy(value,&(*it).second);
	*found = true;

	return S_OK;
}


STDMETHODIMP CCCLSTLMap::append(IDispatch * thatMap, BOOL overwrite)
{
/*
	if(! thatMap)
		return E_POINTER;

	CCL::ICCLSTLMapPtr map(thatMap);

	IUnknownPtr pUnk;
	IEnumVARIANTPtr pNewEnum;
	if( SUCCEEDED(map->get__NewEnum(&pUnk)) && pUnk != NULL)    
	{
		pNewEnum = pUnk;        
		_variant_t varName;
		_variant_t varValue;

		while( pNewEnum->Next(1, &varName, NULL) == S_OK )
		{
			try
			{
				_bstr_t name(varName);
				varValue = map->at[name];
				insert(name,varValue);
			}
			catch(_com_error& e)
			{
				return e.Error();
			}
		}    
	}
*/
	return S_OK;
}

STDMETHODIMP CCCLSTLMap::render(long rtype, BSTR * text)
{
	if( rtype != DUMPTRACE)
		return AtlReportError(	GetObjectCLSID(), 
								"Sorry, that rendering functionality is not implemented.",
								IID_ICCLSTLMap,
								E_INVALIDARG);

	ostrstream os;
	os << "CCLSTLMap[" << m_contents.size() << "]: ";
	ContainerTypeIterator	it;
	for(it=m_contents.begin();it != m_contents.end();it++)
	{
		os << "(";
		os << (const char*)(*it).first;
		os << "->";
		dumpValue((*it).second,os);
		os << ") ";
	}
	os << ends;
	CComBSTR b = os.str();
	os.rdbuf()->freeze(0);
	*text = b.Detach();
	return S_OK;
}
