// CCLSTLVector.cpp : Implementation of CCCLSTLVector
#include "stdafx.h"
#include "CCL.h"
#include "CCLSTLVector.h"
#include "CCLUtility.h"

/////////////////////////////////////////////////////////////////////////////
// CCCLSTLVector

#include "StreamHelpers.h"
#include <comdef.h>

STDMETHODIMP CCCLSTLVector::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ICCLSTLVector,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}
// IPersistStream implementation

STDMETHODIMP CCCLSTLVector::IsDirty()
{
	return S_OK;
}

STDMETHODIMP CCCLSTLVector::Load(IStream* pStm)
{
	// Stream in the file version
	short versionMajor;
	short versionMinor;
	try
	{
		*pStm >> versionMajor;
		*pStm >> versionMinor;
		if( m_versionMajor == versionMajor && m_versionMinor == versionMinor)
		{
			// TODO: Load your stuff here
			long size;  
			*pStm >> size;
			m_contents.resize(size);
			for ( long i=0; i<size; i++ )
			{
				*pStm >> m_contents[i];	// NOTE: your ContentType must have streaming operators defined for it !
										// For std::map<TYPE1,TYPE2> you will have to stream in both the key
										// and the value and insert them into the map	
			}
			
			return S_OK;
		}
		else
			return E_FAIL;
	}
	catch(_com_error& e)
	{
		return e.Error();
	}
	return E_FAIL;
}

STDMETHODIMP CCCLSTLVector::Save(IStream* pStm, BOOL fClearDirty)
{
	try
	{
		*pStm << m_versionMajor; 
		*pStm << m_versionMinor; 
// TODO: Save your stuff here
		long size = m_contents.size();  
		*pStm << size;

		ContainerTypeIterator it;
		for (it=m_contents.begin();it!=m_contents.end();it++)
		{
			*pStm << (*it);			// NOTE: your ContentType must have streaming operators defined for it !
											// For std::map<TYPE1,TYPE2> you will have to stream out both the key
											// and the value
		}
		return S_OK;
	}
	catch(_com_error& e)
	{
		return e.Error();
	}
}

STDMETHODIMP CCCLSTLVector::GetSizeMax(ULARGE_INTEGER* pcbSize)
{
	pcbSize->QuadPart = sizeof( m_versionMajor ) +
						sizeof( m_versionMinor );		
	long size = m_contents.size();
	pcbSize->QuadPart += sizeof( size  ) + 
						size * sizeof(ContentType);
	ContainerTypeIterator it;
	for(it = m_contents.begin();it != m_contents.end(); it++)
	{
		pcbSize->QuadPart += getVariantMarshalSize((*it));		// TODO: 
																// Code assumes ContentType is a VARIANT
																// You will have to calcualte your size here
																// for maps you will have to add the size of both items
	}
	return S_OK;
}

STDMETHODIMP CCCLSTLVector::GetClassID(CLSID *pClassID)
{
	*pClassID = GetObjectCLSID();
	return S_OK;
}	

STDMETHODIMP CCCLSTLVector::get__NewEnum(LPUNKNOWN * pVal)
{
	if (!pVal) return E_POINTER;
	*pVal = 0;
	typedef CComObject< CComEnum<IEnumVARIANT, &IID_IEnumVARIANT, VARIANT,_Copy<VARIANT> > > CComEnumVARIANT;
	VARIANT *pav = new VARIANT[m_contents.size()];
	int j = 0;
	try
	{
		for (ContainerTypeIterator i = m_contents.begin();i != m_contents.end(); ++i)
		{
			::VariantInit(&pav[j]);
			// TODO: write your own conversion here if necessary
			CComVariant	v = *i;		// assume that *i can be converted to a variant
			v.Detach(&pav[j++]);
		}                
	}
	catch (const _com_error& x)                
	{
		for (int i = 0; i < (j-1); ++i)                        
		{
			::VariantClear(&pav[i]);
		}
		delete [] pav;
		return x.Error();                
	}
	CComEnumVARIANT *pEnum = 0;
	HRESULT hr = CComEnumVARIANT::CreateInstance(&pEnum);
	if (hr == S_OK)                
	{
		hr = pEnum->Init(&pav[0], &pav[m_contents.size()], 0, AtlFlagTakeOwnership);
		if (hr == S_OK)                        
		{
			hr = pEnum->QueryInterface(IID_IUnknown, (void **) pVal);
		}                
	}
	return hr;        
}

HRESULT 
CCCLSTLVector::bounds_check(long key,ContainerTypeIterator& it)
{
	if(key > 0 && key < m_contents.size())
	{
		it = m_contents.begin() + key;
	}
	else
	{
		_variant_t i(key);
		i.ChangeType(VT_BSTR);
		_bstr_t msg("CCCLSTLVector[");
		msg += _bstr_t(i);
		msg+="]: no such entry.";
		return AtlReportError(	GetObjectCLSID(), 
								(const char*)msg,
								IID_ICCLSTLVector,
								E_INVALIDARG);
	}
	return S_OK;
}

STDMETHODIMP CCCLSTLVector::get_Value(long key, VARIANT * pVal)
{
	return get_at(key,pVal);
}

STDMETHODIMP CCCLSTLVector::put_Value(long key, VARIANT newVal)
{
	return put_at(key,newVal);
}

STDMETHODIMP CCCLSTLVector::get_Count(long * pVal)
{
	return size(pVal);
}


STDMETHODIMP CCCLSTLVector::size(long * sz)
{
	if(sz == NULL) return E_POINTER;

	*sz = m_contents.size();
	return S_OK;
}

STDMETHODIMP CCCLSTLVector::resize(long sz)
{
	m_contents.resize(sz);
	return S_OK;
}


STDMETHODIMP CCCLSTLVector::push_back(VARIANT var)
{
	m_contents.push_back(CComVariant(var));
	return S_OK;
}

STDMETHODIMP CCCLSTLVector::get_at(long index, VARIANT * pVal)
{
	if(pVal == NULL) return E_POINTER;

	ContainerTypeIterator	it;
	HRESULT hr = bounds_check(index,it);
	if(FAILED(hr))
		return hr;

	hr = ::VariantCopy(pVal,&(*it));

	return hr;
}

STDMETHODIMP CCCLSTLVector::put_at(long index, VARIANT newVal)
{
	ContainerTypeIterator	it;
	HRESULT hr = bounds_check(index,it);
	if(FAILED(hr))
		return hr;

	m_contents[index] = newVal;

	return S_OK;
}

STDMETHODIMP CCCLSTLVector::clear()
{
	m_contents.clear();
	return S_OK;
}

STDMETHODIMP CCCLSTLVector::empty(BOOL* isEmpty)
{
	if(isEmpty == NULL) return E_POINTER;

	*isEmpty = m_contents.empty();
	return S_OK;
}

STDMETHODIMP CCCLSTLVector::erase(long begin)
{
	ContainerTypeIterator	it;
	HRESULT hr = bounds_check(begin,it);
	if(FAILED(hr))
		return hr;
	m_contents.erase(m_contents.begin()+begin);
	
	return S_OK;
}

STDMETHODIMP CCCLSTLVector::reserve(long res)
{
	m_contents.reserve(res);	
	return S_OK;
}

STDMETHODIMP CCCLSTLVector::capacity(long * cap)
{
	if(cap == NULL) return E_POINTER;

	*cap = m_contents.capacity();
	return S_OK;
}

STDMETHODIMP CCCLSTLVector::max_size(long * max)
{
	if(max == NULL) return E_POINTER;

	*max = m_contents.max_size();
	return S_OK;
}

STDMETHODIMP CCCLSTLVector::render(long rtype,BSTR * text)
{
	if( rtype != DUMPTRACE)
		return AtlReportError(	GetObjectCLSID(), 
								"Sorry, that rendering functionality is not implemented.",
								IID_ICCLSTLMap,
								E_INVALIDARG);
	ostrstream os;
	os << "CCLSTLVector[" << m_contents.size() << "]: ";		
	for(int i=0;i<m_contents.size();i++)
	{
		os << "(";
		dumpValue(m_contents[i],os);
		os << ") ";
	}
	os << ends;
	CComBSTR b = os.str();
	os.rdbuf()->freeze(0);
	*text = b.Detach();
	return S_OK;
}
