/*******************************************************************************************
 * Visual Source Safe Version Control Information
 *******************************************************************************************
 * $History: COMSTLUtility.h $
 *******************************************************************************************/
#include <FFS/MSF/Utility/StdInclude.h>
#include <FFS/MSF/Utility/COMSTLUtility.h>
#include <FFS/MSF/Utility/ADO/ADOUtility.h>
#include <strstrea.h>
#include <atlbase.h>
#include <FFS/MSF/Utility/Logging/Tracing.h>
#include <FFS/MSF/Utility/enum_iterator.h>

#import "ComSTL.tlb" rename_namespace("ComSTL")

#ifndef MSF_NO_NAMESPACE
namespace MSF
{
#endif



//#include <FFS/MSF/Utility/Logging/Tracing.h>

/*
#define CASEVARIANT(X) case X : return #X; break
const char* variantType(VARTYPE type)
{
	switch(type)
	{
		CASEVARIANT(VT_EMPTY);
		CASEVARIANT(VT_NULL);
		CASEVARIANT(VT_I2);
		CASEVARIANT(VT_I4);
		CASEVARIANT(VT_R4);
		CASEVARIANT(VT_R8);
		CASEVARIANT(VT_CY);
		CASEVARIANT(VT_DATE);
		CASEVARIANT(VT_BSTR);
		CASEVARIANT(VT_DISPATCH);
		CASEVARIANT(VT_ERROR);
		CASEVARIANT(VT_BOOL);
		CASEVARIANT(VT_VARIANT);
		CASEVARIANT(VT_UNKNOWN);
		CASEVARIANT(VT_DECIMAL);
		CASEVARIANT(VT_I1);
		CASEVARIANT(VT_UI1);
		CASEVARIANT(VT_UI2);
		CASEVARIANT(VT_UI4);
		CASEVARIANT(VT_I8);
		CASEVARIANT(VT_UI8);
		CASEVARIANT(VT_INT);
		CASEVARIANT(VT_UINT);
		CASEVARIANT(VT_VOID);
		CASEVARIANT(VT_HRESULT);
		CASEVARIANT(VT_PTR);
		CASEVARIANT(VT_SAFEARRAY);
		CASEVARIANT(VT_CARRAY);
		CASEVARIANT(VT_USERDEFINED);
		CASEVARIANT(VT_LPSTR);
		CASEVARIANT(VT_LPWSTR);
		CASEVARIANT(VT_FILETIME);
		CASEVARIANT(VT_BLOB);
		CASEVARIANT(VT_STREAM);
		CASEVARIANT(VT_STORAGE);
		CASEVARIANT(VT_STREAMED_OBJECT);
		CASEVARIANT(VT_STORED_OBJECT);
		CASEVARIANT(VT_BLOB_OBJECT);
		CASEVARIANT(VT_CF);
		CASEVARIANT(VT_CLSID);
		CASEVARIANT(VT_VECTOR);
		CASEVARIANT(VT_ARRAY);
		CASEVARIANT(VT_BYREF);
		default:
			return "Unknown";
	}
}




_bstr_t asBstr(ostrstream& os)
{
	os << ends;
	_bstr_t bs(os.str());
	os.rdbuf()->freeze(0);
	os.seekp(0);
	return bs;
}

void
decode(	IErrorInfo* errorInfo,
		ostream& os)
{
	if(errorInfo)
	{
		BSTR src = NULL,desc = NULL,hlp = NULL;
		DWORD hlpCtx = 0;
		errorInfo->GetDescription(&desc);
		errorInfo->GetSource(&src);
		errorInfo->GetHelpFile(&hlp);
		errorInfo->GetHelpContext(&hlpCtx);
		_bstr_t s(src,false),d(desc,false),h(hlp,false);
		
		os << "DESCRIPTION="	<< SAFEBSTR((const char*)d) << ";";
		os << "HELPCONTEXT="	<< hlpCtx << ";";
		os << "HELPFILE="		<< SAFEBSTR((const char*)h) << ";";
		os << "SOURCE="			<< SAFEBSTR((const char*)s) << ";";

		//	os << "GUID="			<< e.GUID() << ";";
	}
}


_bstr_t asBstr(_com_error& e)
{
	ostrstream msg;
    try
    {
		msg << "ERROR="			<< e.Error() << ";";
		msg << "ERRORMESSAGE="	<< SAFEBSTR(e.ErrorMessage()) << ";";
		IErrorInfo* errorInfo = e.ErrorInfo();
		if(errorInfo)
		{
			decode(errorInfo,msg);
			errorInfo->Release();
		}
    }
	catch(...)
	{
		msg << "error -> Caught UNKNOWN EXCEPTION."; 
	}
	return asBstr(msg);
}

*/


_bstr_t depthString(int depth,
					char c)
{
	ostrstream os;
	for(int dc=0;dc<depth;dc++)
		os << c;
	_bstr_t ds(MSF::CTraceFormat::asBstr(os));
	return ds;
}


void
dumpValue(	const _bstr_t& name,
			const _variant_t& varValue,
			ostream& os,
			const char* sep,
			int depth)
{
	os << " {" << MSF::CTraceFormat::variantType(varValue.vt) << "} ";

	if(varValue.vt == VT_DISPATCH)
	{
		os << "-> ";
		IDispatch*	disp = varValue.pdispVal;

		ComSTL::ISTLVectorPtr vector;
		if(SUCCEEDED(disp->QueryInterface(__uuidof(ComSTL::ISTLVector),(void**)&vector)))
		{
			dumpSTLVector(name,vector,os,sep,depth+1);
			return;
		}

		ComSTL::ISTLMapPtr map;
		if(SUCCEEDED(disp->QueryInterface(__uuidof(ComSTL::ISTLMap),(void**)&map)))
		{
			dumpSTLMap(name,map,os,sep,depth+1);
			return;
		}

		_RecordsetPtr recordSet;
		if(SUCCEEDED(disp->QueryInterface(__uuidof(_Recordset),(void**)&recordSet)))
		{
			os << "_Recordset  " << name << "[] -> ";
			MSF::CADOUtility::dumpLayout(recordSet,os);
//			MSF::CADOUtility::dumpRecords(recordSet,os);
			os << sep;
			return;
		}

		os << "Unknown" << sep;
	}
	else
	{
		_variant_t var = varValue;
		var.ChangeType(VT_BSTR);
		_bstr_t bv(var);
		const char* s = bv;
		os << bv << sep;
	}
}

void 
dumpSTLVector(const _bstr_t& listName,
			   ComSTL::ISTLVectorPtr& list,
			   ostream& os,
			   const char* sep,
			   int depth)
{
	_bstr_t ds = depthString(depth,' ');

	int c = list->Count;
	os << "ISTLVector  " << listName << "(" << c << ") ->" << sep;

	for(int i=0;i<c;i++)
	{
		try
		{
			_variant_t varValue = list->at[i];

			ostrstream itemName;
			itemName << listName << "(" << i << ")";
			_bstr_t bitemName(MSF::CTraceFormat::asBstr(itemName));

			os << ds << bitemName << " = ";
			dumpValue(bitemName,varValue,os,sep,depth);
		}
		catch(_com_error&)
		{
			os << "FAILED" << sep;
		}
	}
}




void 
dumpSTLMap(const _bstr_t& mapName,
			   ComSTL::ISTLMapPtr& map,
			   ostream& os,
			   const char* sep,
			   int depth)
{
	_bstr_t ds = depthString(depth,' ');

	os << ds << "ISTLMap  " << mapName << "[" << map->GetCount() << "] ->" << sep;

	IUnknownPtr pUnk;
	IEnumVARIANTPtr pNewEnum;
	if( SUCCEEDED(map->get__NewEnum(&pUnk)) && pUnk != NULL)    
	{
		pNewEnum = pUnk;        
		_variant_t varName;
		_variant_t varValue;

		while( pNewEnum->Next(1, &varName, NULL) == S_OK )
		{
			try
			{
				_bstr_t name(varName);
				os << ds << mapName << "[" << name << "] = ";

				varValue = map->at[name];

				dumpValue(name,varValue,os,sep,depth);
			}
			catch(_com_error& e)
			{
				os << "FAILED" << MSF::CTraceFormat::asBstr(e) << sep;
			}
		}    
	}
}


_bstr_t mapAsBstr(const char* name,ComSTL::ISTLMapPtr& map)
{
	ostrstream os;
	dumpSTLMap(name,map,os,"\n");
	return MSF::CTraceFormat::asBstr(os);
//	return asBstr(os);
}

_bstr_t vectorAsBstr(const char* name,ComSTL::ISTLVectorPtr& vector)
{
	ostrstream os;
	dumpSTLVector(name,vector,os,"\n");
	return MSF::CTraceFormat::asBstr(os);
}

HRESULT mapAsBlob(ComSTL::ISTLMapPtr& map, _variant_t& blob)
{
	try
	{
		IStreamPtr istream;
		_CHECK_ERROR(CreateStreamOnHGlobal(NULL,
										   false,
										   &istream));

		IPersistStreamPtr pstream = map;

		pstream->Save(istream, true);

		STATSTG statstg;
		_CHECK_ERROR(istream->Stat(&statstg, STATFLAG_NONAME));

		ULARGE_INTEGER size = statstg.cbSize;
		unsigned long bufferSize = size.QuadPart;

		HGLOBAL hglobal = 0;
		_CHECK_ERROR(GetHGlobalFromStream(istream, &hglobal));

		unsigned char* buffer = (unsigned char*)GlobalLock(hglobal);
		GlobalUnlock(hglobal);

		SAFEARRAY FAR* psa = SafeArrayCreateVector(VT_UI1, 0, bufferSize);
		if(psa != NULL)    
		{
			psa->pvData = (void HUGEP*)buffer;
			
			_variant_t var;
			var.vt = VT_ARRAY | VT_UI1;
			var.parray = psa;

			blob = var;        
		}

		return S_OK;
	}
	catch( _com_error& e)
	{
		_CAUGHT_COMERROR_RETURN(e);
	}
}

HRESULT blobAsMap(ComSTL::ISTLMapPtr& map, 
				  _variant_t& varBuffer,
				  unsigned long bufferSize)
{
	try
	{
		_CHECK_ERROR(map.CreateInstance(__uuidof(ComSTL::CSTLMap)));

		if (bufferSize > 0)      
		{
			varBuffer.ChangeType(VT_ARRAY | VT_UI1);

			HGLOBAL hglobal = GlobalAlloc(GPTR, bufferSize);
			if (hglobal)
			{
				unsigned char* buffer = (unsigned char*)GlobalLock(hglobal);
				memcpy(buffer, varBuffer.parray->pvData, bufferSize);
				GlobalUnlock(hglobal);

				IStreamPtr istream;
				_CHECK_ERROR(CreateStreamOnHGlobal(hglobal,
												   false,
												   &istream));

				IPersistStreamPtr pstream(map);
				pstream->Load(istream);
			}
		}

		return S_OK;
	}
	catch( _com_error& e)
	{
		_CAUGHT_COMERROR_RETURN(e);
	}
}

#ifndef MSF_NO_NAMESPACE
};
#endif
