#ifndef __CCCLUTILITY_H_
#define __CCCLUTILITY_H_
#include <comdef.h>

class ostrstream;
class ostream;


inline const char* variantType(VARTYPE type)
{
	switch(type)
	{
		CASEVARIANT(VT_EMPTY);
		CASEVARIANT(VT_NULL);
		CASEVARIANT(VT_I2);
		CASEVARIANT(VT_I4);
		CASEVARIANT(VT_R4);
		CASEVARIANT(VT_R8);
		CASEVARIANT(VT_CY);
		CASEVARIANT(VT_DATE);
		CASEVARIANT(VT_BSTR);
		CASEVARIANT(VT_DISPATCH);
		CASEVARIANT(VT_ERROR);
		CASEVARIANT(VT_BOOL);
		CASEVARIANT(VT_VARIANT);
		CASEVARIANT(VT_UNKNOWN);
		CASEVARIANT(VT_DECIMAL);
		CASEVARIANT(VT_I1);
		CASEVARIANT(VT_UI1);
		CASEVARIANT(VT_UI2);
		CASEVARIANT(VT_UI4);
		CASEVARIANT(VT_I8);
		CASEVARIANT(VT_UI8);
		CASEVARIANT(VT_INT);
		CASEVARIANT(VT_UINT);
		CASEVARIANT(VT_VOID);
		CASEVARIANT(VT_HRESULT);
		CASEVARIANT(VT_PTR);
		CASEVARIANT(VT_SAFEARRAY);
		CASEVARIANT(VT_CARRAY);
		CASEVARIANT(VT_USERDEFINED);
		CASEVARIANT(VT_LPSTR);
		CASEVARIANT(VT_LPWSTR);
		CASEVARIANT(VT_FILETIME);
		CASEVARIANT(VT_BLOB);
		CASEVARIANT(VT_STREAM);
		CASEVARIANT(VT_STORAGE);
		CASEVARIANT(VT_STREAMED_OBJECT);
		CASEVARIANT(VT_STORED_OBJECT);
		CASEVARIANT(VT_BLOB_OBJECT);
		CASEVARIANT(VT_CF);
		CASEVARIANT(VT_CLSID);
		CASEVARIANT(VT_VECTOR);
		CASEVARIANT(VT_ARRAY);
		CASEVARIANT(VT_BYREF);
		default:
			return "Unknown";
	}
}



inline void dumpValue(	const _bstr_t& name,
						const _variant_t& varValue,
						ostream& os,
						const char* sep = "\n",
						int depth = 0)
{
	os << " {" << variantType(varValue.vt) << "} ";

	if(varValue.vt == VT_DISPATCH)
	{
		os << "-> ";
		IDispatch*	disp = varValue.pdispVal;

		ComSTL::ISTLVectorPtr vector;
		if(SUCCEEDED(disp->QueryInterface(__uuidof(ComSTL::ISTLVector),(void**)&vector)))
		{
			dumpSTLVector(name,vector,os,sep,depth+1);
			return;
		}

		ComSTL::ISTLMapPtr map;
		if(SUCCEEDED(disp->QueryInterface(__uuidof(ComSTL::ISTLMap),(void**)&map)))
		{
			dumpSTLMap(name,map,os,sep,depth+1);
			return;
		}

/*
		_RecordsetPtr recordSet;
		if(SUCCEEDED(disp->QueryInterface(__uuidof(_Recordset),(void**)&recordSet)))
		{
			os << "_Recordset  " << name << "[] -> ";
			dumpRecordsetLayout(recordSet,os);
			os << sep;
			return;
		}
*/
		os << "Unknown" << sep;
	}
	else
	{
		_variant_t var = varValue;
		var.ChangeType(VT_BSTR);
		_bstr_t bv(var);
		const char* s = bv;
		os << bv << sep;
	}
}

inline void dumpSTLVector(	const _bstr_t& listName,
					COMContLib::ICCLSTLVectorPtr& list,
					ostream& os,
					const char* sep = "\n",
					int depth = 0)
{
	_bstr_t ds = depthString(depth,' ');

	int c = list->Count;
	os << "ISTLVector  " << listName << "(" << c << ") ->" << sep;

	for(int i=0;i<c;i++)
	{
		try
		{
			_variant_t varValue = list->at[i];

			ostrstream itemName;
			itemName << listName << "(" << i << ")";
			_bstr_t bitemName(MSF::CTraceFormat::asBstr(itemName));

			os << ds << bitemName << " = ";
			dumpValue(bitemName,varValue,os,sep,depth);
		}
		catch(_com_error&)
		{
			os << "FAILED" << sep;
		}
	}
}

inline void dumpSTLMap(	const _bstr_t& mapName,
					COMContLib::ICCLSTLMapPtr& map,
					ostream& os,
					const char* sep = "\n",
					int depth = 0)
{
	_bstr_t ds = depthString(depth,' ');

	os << ds << "ISTLMap  " << mapName << "[" << map->GetCount() << "] ->" << sep;

	IUnknownPtr pUnk;
	IEnumVARIANTPtr pNewEnum;
	if( SUCCEEDED(map->get__NewEnum(&pUnk)) && pUnk != NULL)    
	{
		pNewEnum = pUnk;        
		_variant_t varName;
		_variant_t varValue;

		while( pNewEnum->Next(1, &varName, NULL) == S_OK )
		{
			try
			{
				_bstr_t name(varName);
				os << ds << mapName << "[" << name << "] = ";

				varValue = map->at[name];

				dumpValue(name,varValue,os,sep,depth);
			}
			catch(_com_error& e)
			{
				os << "FAILED" << MSF::CTraceFormat::asBstr(e) << sep;
			}
		}    
	}
}


inline _bstr_t	asBstr(	ostrstream& os, 
						bool terminate, 
						bool reset)
{
	if(terminate)
		os << ends;

	_bstr_t str( os.str() );
	os.rdbuf()->freeze(0);

	if(reset)
		os.seekp(0);

	return str;
}

inline _bstr_t depthString(	int depth,
							char c = ' ')
{
	ostrstream os;
	for(int dc=0;dc<depth;dc++)
		os << c;
	asBstr(os);
	return ds;
}



inline  _bstr_t mapAsBstr(const char* name,COMContLib::ICCLSTLMapPtr& map)
{
	ostrstream os;
	dumpSTLMap(name,map,os,"\n");
	return asBstr(os);
}

inline  _bstr_t vectorAsBstr(const char* name,COMContLib::ICCLSTLVectorPtr& vector)
{
	ostrstream os;
	dumpSTLVector(name,vector,os,"\n");
	return MSF::CTraceFormat::asBstr(os);
}




#endif