// MBVContainerPage_impl.h : Declaration of the CMBVContainerPage

#ifndef __MBVCONTAINERPAGE_H_
#define __MBVCONTAINERPAGE_H_

#include "resource.h"       // main symbols
#include "WoA_SymbolMap.h"      // the ISymbolMap interface

EXTERN_C const CLSID CLSID_MBVContainerPage;

/////////////////////////////////////////////////////////////////////////////
// CMBVContainerPage
class ATL_NO_VTABLE CMBVContainerPage :
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CMBVContainerPage, &CLSID_MBVContainerPage>,
	public IPropertyPageImpl<CMBVContainerPage>,
	public CDialogImpl<CMBVContainerPage>
{
public:
	CMBVContainerPage() 
	{
		m_dwTitleID = IDS_TITLEMBVContainerPage;
		m_dwHelpFileID = IDS_HELPFILEMBVContainerPage;
		m_dwDocStringID = IDS_DOCSTRINGMBVContainerPage;
		m_bActivated = false;
	}

	enum {IDD = IDD_MBVCONTAINERPAGE};

DECLARE_REGISTRY_RESOURCEID(IDR_MBVCONTAINERPAGE)

BEGIN_COM_MAP(CMBVContainerPage) 
	COM_INTERFACE_ENTRY_IMPL(IPropertyPage)
END_COM_MAP()

BEGIN_MSG_MAP(CMBVContainerPage)
	CHAIN_MSG_MAP(IPropertyPageImpl<CMBVContainerPage>)
END_MSG_MAP()

	bool m_bActivated;

	STDMETHOD(Activate)(HWND hWndParent, LPCRECT prc, BOOL bModal)
	{
		ATLTRACE(_T("CMBVContainerPage::Activate\n"));
		m_bActivated = true;
		HRESULT hres = IPropertyPageImpl<CMBVContainerPage>::Activate(hWndParent, prc, bModal);
		if (SUCCEEDED(hres))
		{
			// Loop through any objects and keep the first one that has an ISymbolMap interface
			for (UINT i = 0; i < m_nObjects; i++)
			{
				ISymbolMap* pMap = NULL;
				if (SUCCEEDED(m_ppUnk[i]->QueryInterface(__uuidof(ISymbolMap), (void**)&pMap)))
				{
					HRESULT hresActivate = OnPageActivate(pMap);
					pMap->Release();
					return hresActivate;
				}
			}
		}
		return hres; 
	}
	STDMETHOD(Apply)(void)
	{
		ATLTRACE(_T("CMBVContainerPage::Apply\n"));
		// Loop through any objects and keep the first one that has an ISymbolMap interface
		for (UINT i = 0; i < m_nObjects; i++)
		{
			ISymbolMap* pMap = NULL;
			if (SUCCEEDED(m_ppUnk[i]->QueryInterface(__uuidof(ISymbolMap), (void**)&pMap)))
			{
				HRESULT hresApply = S_OK;
				if (m_bActivated)
					 hresApply = OnPageApply(pMap);
				pMap->Release();
				if (SUCCEEDED(hresApply))
					m_bDirty = FALSE;
				return hresApply;
			}
		}
		return E_UNEXPECTED;
	}

	STDMETHOD(Deactivate)(void)
	{
		ATLTRACE(_T("CMBVContainerPage::Deactivate\n"));
		if (SUCCEEDED(OnPageDeactivate()))
		{
			return IPropertyPageImpl<CMBVContainerPage>::Deactivate();
		}
		else
		{
			IPropertyPageImpl<CMBVContainerPage>::Deactivate();
			return E_UNEXPECTED;
		}
	}

	// ******************************************************************************* //

	// To use these helpers, create a string resource with the same ID as the control
	// The string resource will be used as the name of the symbol and the contents of the
	// control as its value.

	HRESULT SetSymbolFromCheckBox(int nID, ISymbolMap* pMap)
	{
		// Call this function to set a symbol from a check box control
		const int nBufferMax = 256;
		LPTSTR pBuffer = new TCHAR [nBufferMax];
		if (!LoadString(_Module.GetResourceInstance(), nID, pBuffer, nBufferMax))
			return E_FAIL;
      
		CComBSTR bstrSymbol(pBuffer);
		delete [] pBuffer;
		return pMap->Set( bstrSymbol,
			CComBSTR(IsDlgButtonChecked(nID) ? "TRUE" : "FALSE") );
	}

	HRESULT SetSymbolFromTextBox(int nID, ISymbolMap* pMap)
	{
		// Call this function to set a symbol from a text box control
		const int nBufferMax = 256;
		LPTSTR pBuffer = new TCHAR [nBufferMax];
		if (!LoadString(_Module.GetResourceInstance(), nID, pBuffer, nBufferMax))
			return E_FAIL;
      
		CComBSTR bstrSymbol(pBuffer);
		delete [] pBuffer;
		CComBSTR bstrValue;
		if (!GetDlgItemText(nID, bstrValue.m_str))
			return E_FAIL;
		return pMap->Set( bstrSymbol,bstrValue );
	}

	// ******************************************************************************* //

   
	// We can guarantee that pMap is a valid pointer to an ISymbolMap interface for both the
	// OnPageAxxxx() functions.
	// In fact, neither of these functions will be called if we can't get a valid pointer.

	HRESULT OnPageActivate(ISymbolMap* pMap)
	{
		// TODO - get symbols for initializations from the Symbol Map if necessary
		// and perform other initializations
		// DO NOT release or store pMap
		// Return one of E_OUTOFMEMORY, E_UNEXPECTED, S_OK
		SendDlgItemMessage(IDC_ISCONTAINER,BM_SETCHECK,BST_UNCHECKED,0);
		SetDlgItemText(IDC_INDEXTYPE, "long");
		return S_OK;
	}

	HRESULT OnPageApply(ISymbolMap* pMap)
	{
		// TODO - set symbols in the Symbol Map (you can use SetSymbolFromXXXBox() to help)
		// DO NOT release or store pMap
		// Return one of E_OUTOFMEMORY, E_UNEXPECTED, S_FALSE, S_OK
		SetSymbolFromCheckBox(IDC_ISCONTAINER, pMap);
		SetSymbolFromTextBox(IDC_INDEXTYPE, pMap);
		return S_OK;
	}

	HRESULT OnPageDeactivate()
	{
		// TODO - perform any cleanup necessary
		// Return one of E_UNEXPECTED, S_OK
		return S_OK;
	}

};

#endif //__MBVCONTAINERPAGE_H_
